%define anolis_release 3

%undefine _package_note_flags
Name:           ocaml-lwt
Version:        5.4.2
Release:        %{anolis_release}%{?dist}
Summary:        OCaml lightweight thread library

# The openssl linking exception is granted.
License:        LGPLv2+ with exceptions
URL:            https://ocsigen.org/lwt
Source0:        https://github.com/ocsigen/lwt/archive/refs/tags/%{version}.tar.gz
Patch001:	0001-fix-test-error-for-loongarch64.patch

BuildRequires:  ocaml >= 4.02.0
BuildRequires:  ocaml-findlib
BuildRequires:  dune
BuildRequires:  ocaml-dune-devel
BuildRequires:  ocaml-cppo
BuildRequires:  ocaml-result-devel
BuildRequires:  ocaml-seq-devel
BuildRequires:  ocaml-bisect-ppx-devel

# lwt.unix dependencies
BuildRequires:  ocaml-mmap-devel
BuildRequires:  ocaml-ocplib-endian-devel

# lwt_luv dependencies.
BuildRequires:  ocaml-luv-devel

# lwt_react dependencies.
BuildRequires:  ocaml-react-devel

# lwt_ppx dependencies.
BuildRequires:  ocaml-ppxlib-devel >= 0.16.0

BuildRequires:  glib2-devel
BuildRequires:  libev-devel

%description
Lwt is a lightweight thread library for Objective Caml.  This library
is part of the Ocsigen project.

%package        devel
Summary:        Development files for %{name}
Requires:       %{name} = %{version}-%{release}
Requires:       ocaml-mmap-devel
Requires:       ocaml-ocplib-endian-devel
Requires:       ocaml-result-devel
Requires:       ocaml-seq-devel
Requires:       libev-devel

%description    devel
The %{name}-devel package contains libraries and signature files for
developing applications that use %{name}.

%package        luv
Summary:        Libuv engine for lwt
Requires:       %{name} = %{version}-%{release}

%description    luv
This package contains a libuv engine for lwt.

%package        luv-devel
Summary:        Development files for ocaml-lwt-luv

Requires:       %{name}-luv = %{version}-%{release}
Requires:       %{name}-devel = %{version}-%{release}
Requires:       ocaml-luv-devel
Requires:       ocaml-result-devel

%description    luv-devel
The %{name}-luv-devel package contains libraries and signature files for
developing applications that use %{name}-luv.

%package        react
Summary:        Helpers for using React with Lwt
Requires:       %{name} = %{version}-%{release}

%description    react
Helpers for using React with Lwt.

%package        react-devel
Summary:        Development files for ocaml-lwt-react

Requires:       %{name}-react = %{version}-%{release}
Requires:       %{name}-devel = %{version}-%{release}
Requires:       ocaml-react-devel

%description    react-devel
The %{name}-react-devel package contains libraries and signature files for
developing applications that use %{name}-react.

%package        ppx
Summary:        PPX syntax for Lwt
Requires:       %{name} = %{version}-%{release}

%description    ppx
PPX syntax for Lwt, providing something similar to async/await from JavaScript.

%package        ppx-devel
Summary:        Development files for ocaml-lwt-ppx

Requires:       %{name}-ppx = %{version}-%{release}
Requires:       %{name}-devel = %{version}-%{release}
Requires:       ocaml-ppxlib-devel

%description    ppx-devel
The %{name}-ppx-devel package contains libraries and signature files for
developing applications that use %{name}-ppx.

%package        doc
Summary:        Documentation files for %{name}
Requires:       %{name} = %{EVR}
BuildArch:      noarch

%description    doc
The %{name}-doc package contains documentation files for %{name}.

%prep
%autosetup -p1 -n lwt-%{version}

%ifarch riscv64
# It looks like one test fails.
# Actually, it looks like all the "mcast" tests fail in koji.
# They should probably be disabled via a patch, but this works for now.
sed 's,test_mcast "mcast-join-loop" true true;,(*test_mcast "mcast-join-loop" true true;*),' -i test/unix/test_mcast.ml
sed 's,test_mcast "mcast-join-noloop" true false;,(*test_mcast "mcast-join-noloop" true false;*),' -i test/unix/test_mcast.ml
sed 's,test_mcast "mcast-nojoin-loop" false true;,(*test_mcast "mcast-nojoin-loop" false true;*),' -i test/unix/test_mcast.ml
sed 's,test_mcast "mcast-nojoin-noloop" false false;,(*test_mcast "mcast-nojoin-noloop" false false;*),' -i test/unix/test_mcast.ml
%endif

%build
# Enable libev and pthread.
dune exec src/unix/config/discover.exe -- --save \
     --use-libev true --use-pthread true
dune build %{?_smp_mflags} --profile=release --verbose

# Relink the stublib with RPM_LD_FLAGS
cd _build/default/src/unix
ocamlmklib -g -ldopt "$RPM_LD_FLAGS" -lev -lpthread -lm -o lwt_unix_stubs \
  $(ar t liblwt_unix_stubs.a)
cd -

%install
dune install --profile=release --destdir %{buildroot}

# Remove spurious dune-installed documentation.
rm -rf %{buildroot}/%{_prefix}/doc

%generate_compatibility_deps

%check
dune runtest --profile=release

%files
%license LICENSE.md
%dir %{_libdir}/ocaml/lwt/
%dir %{_libdir}/ocaml/lwt/unix/
%{_libdir}/ocaml/lwt/META
%{_libdir}/ocaml/lwt/dune-package
%{_libdir}/ocaml/lwt/opam
%{_libdir}/ocaml/lwt/*.cma
%{_libdir}/ocaml/lwt/*.cmi
%{_libdir}/ocaml/lwt/unix/*.cma
%{_libdir}/ocaml/lwt/unix/*.cmi
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/lwt/*.cmxs
%{_libdir}/ocaml/lwt/unix/*.cmxs
%endif
%{_libdir}/ocaml/stublibs/*.so
%dir %{abidir}
%{abidir}/dlllwt_unix_stubs.dump

%files devel
%doc CHANGES README.md
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/lwt/*.a
%{_libdir}/ocaml/lwt/*.cmxa
%{_libdir}/ocaml/lwt/*.cmx
%{_libdir}/ocaml/lwt/unix/*.a
%{_libdir}/ocaml/lwt/unix/*.cmxa
%{_libdir}/ocaml/lwt/unix/*.cmx
%endif
%{_libdir}/ocaml/lwt/*.cmt
%{_libdir}/ocaml/lwt/*.cmti
%{_libdir}/ocaml/lwt/*.ml
%{_libdir}/ocaml/lwt/*.mli
%{_libdir}/ocaml/lwt/unix/*.cmt
%{_libdir}/ocaml/lwt/unix/*.cmti
%{_libdir}/ocaml/lwt/unix/*.h
%{_libdir}/ocaml/lwt/unix/*.ml
%{_libdir}/ocaml/lwt/unix/*.mli

%files luv
%doc CHANGES README.md
%license LICENSE.md
%dir %{_libdir}/ocaml/lwt_luv/
%{_libdir}/ocaml/lwt_luv/META
%{_libdir}/ocaml/lwt_luv/dune-package
%{_libdir}/ocaml/lwt_luv/opam
%{_libdir}/ocaml/lwt_luv/*.cma
%{_libdir}/ocaml/lwt_luv/*.cmi
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/lwt_luv/*.cmxs
%endif

%files luv-devel
%doc CHANGES README.md
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/lwt_luv/*.a
%{_libdir}/ocaml/lwt_luv/*.cmxa
%{_libdir}/ocaml/lwt_luv/*.cmx
%endif
%{_libdir}/ocaml/lwt_luv/*.cmt
%{_libdir}/ocaml/lwt_luv/*.cmti
%{_libdir}/ocaml/lwt_luv/*.ml
%{_libdir}/ocaml/lwt_luv/*.mli

%files react
%doc CHANGES README.md
%license LICENSE.md
%dir %{_libdir}/ocaml/lwt_react/
%{_libdir}/ocaml/lwt_react/META
%{_libdir}/ocaml/lwt_react/dune-package
%{_libdir}/ocaml/lwt_react/opam
%{_libdir}/ocaml/lwt_react/*.cma
%{_libdir}/ocaml/lwt_react/*.cmi
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/lwt_react/*.cmxs
%endif

%files react-devel
%doc CHANGES README.md
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/lwt_react/*.a
%{_libdir}/ocaml/lwt_react/*.cmxa
%{_libdir}/ocaml/lwt_react/*.cmx
%endif
%{_libdir}/ocaml/lwt_react/*.cmt
%{_libdir}/ocaml/lwt_react/*.cmti
%{_libdir}/ocaml/lwt_react/*.ml
%{_libdir}/ocaml/lwt_react/*.mli

%files ppx
%dir %{_libdir}/ocaml/lwt_ppx/
%{_libdir}/ocaml/lwt_ppx/META
%{_libdir}/ocaml/lwt_ppx/dune-package
%{_libdir}/ocaml/lwt_ppx/opam
%{_libdir}/ocaml/lwt_ppx/ppx.exe
%{_libdir}/ocaml/lwt_ppx/*.cma
%{_libdir}/ocaml/lwt_ppx/*.cmi
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/lwt_ppx/*.cmxs
%endif

%files ppx-devel
%doc CHANGES README.md
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/lwt_ppx/*.a
%{_libdir}/ocaml/lwt_ppx/*.cmxa
%{_libdir}/ocaml/lwt_ppx/*.cmx
%endif
%{_libdir}/ocaml/lwt_ppx/*.cmt
%{_libdir}/ocaml/lwt_ppx/*.cmti
%{_libdir}/ocaml/lwt_ppx/*.ml
%{_libdir}/ocaml/lwt_ppx/*.mli

%files doc
%doc CHANGES README.md

%changelog
* Fri Jun 06 2025 Shangtong Guo <guo.shangtong@zte.com.cn> - 5.4.2-3
- fix test error fot riscv64

* Tue Nov 14 2023 Wenlong Zhang <zhangwenlong@loongson.cn> - 5.4.2-2
- fix test error for loongarch64

* Mon Mar 27 2023 Yuanhong Peng <yummypeng@linux.alibaba.com> - 5.4.2-1
- Init from upstream
